"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskExecutor = void 0;
const execa_1 = __importDefault(require("execa"));
const rxjs_1 = require("rxjs");
const common_1 = require("@nnode/common");
const Logging_1 = require("../Logging");
const TaskEntryType_1 = require("../Models/TaskEntryType");
const TaskResultError_1 = require("../Errors/TaskResultError");
class TaskExecutor extends rxjs_1.Subject {
    constructor() {
        super(...arguments);
        this.log = Logging_1.Logger.extend('task-executor');
    }
    execute(entry, options) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const execaOpts = {
                    cwd: options.cwd,
                    detached: entry.type === TaskEntryType_1.TaskEntryType.detached,
                    env: options.env,
                    shell: entry.type === TaskEntryType_1.TaskEntryType.shell ? '/bin/bash' : false,
                };
                const command = [entry.name, ...entry.args].join(' ');
                this.log.info('execute', command, entry);
                if (options.streaming) {
                    const stream = this.stream(entry, execaOpts);
                    const results = yield stream;
                    this.complete();
                    return this.createResult(entry, results);
                }
                const result = yield this.sync(entry, execaOpts);
                this.complete();
                return result;
            }
            catch (error) {
                return this.bail(error, entry);
            }
        });
    }
    bail(error, entry) {
        this.error(error);
        if (entry.type === TaskEntryType_1.TaskEntryType.nobail) {
            return {
                command: entry.command,
                entry,
                exitCode: 0,
                stderr: error.stderr ? error.stderr.split('\n') : [],
                stdout: error.stdout ? error.stdout.split('\n') : [],
            };
        }
        process.exit(error.exitCode);
    }
    broadcast(result) {
        if (result.exitCode !== 0) {
            this.error(new TaskResultError_1.TaskResultError(result));
        }
        else {
            this.next(result);
        }
        this.log.trace('broadcast', result);
        return result;
    }
    createResult(entry, results) {
        const result = {
            entry,
            command: [entry.name, ...entry.args].join(' '),
            exitCode: results.exitCode,
            stderr: results.stderr ? results.stderr.split('\n') : [],
            stdout: results.stdout ? results.stdout.split('\n') : [],
        };
        return this.broadcast(result);
    }
    stream(entry, options) {
        const child = execa_1.default(entry.name, entry.args, Object.assign(Object.assign({}, options), { stderr: process.stderr, stdin: process.stdin, stdout: process.stdout }));
        if (child.stderr) {
            child.stderr.pipe(process.stderr);
        }
        if (child.stdout) {
            child.stdout.pipe(process.stdout);
        }
        return child;
    }
    sync(entry, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const command = [entry.name].concat(entry.args).join(' ');
            const results = yield execa_1.default.command(command, common_1.Merge(options, {
                stderr: process.stderr,
                stdin: process.stdin,
                stdout: process.stdout,
            }));
            if (results.exitCode !== 0) {
                throw new Error(`executor failed when running: ${command}`);
            }
            return this.createResult(entry, results);
        });
    }
}
exports.TaskExecutor = TaskExecutor;
//# sourceMappingURL=TaskExecutor.js.map